/**
 * JASMINe
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.api;

import org.drools.KnowledgeBase;
import org.drools.builder.KnowledgeBuilder;
import org.drools.runtime.rule.FactHandle;
import org.ow2.jasmine.rules.AbstractDroolsWorkingMemory;

/**
 * A service to manage the working memory.
 * @author Loris Bouzonnet
 */
public interface IDroolsWorkingMemory {

    /**
     * Start the working memory.
     * @throws Exception if the working memory cannot be started
     */
    void start() throws Exception;

    /**
     * Stop the WorkingMemory, with the state STOPPED.
     * @see {@link AbstractDroolsWorkingMemory#stop(DroolsWorkingMemoryState)}
     */
    void stop();

    /**
     * Stop the working memory with a given state (allowed states are
     * STOPPED and IDLE)
     */
    void stop(DroolsWorkingMemoryState state);

    /**
     * Insert a fact in the WorkingMemory
     * @param fact the fact object to insert
     * @return the corresponding FactHandle
     */
    FactHandle insert(Object fact);

    /**
     * Get the KnowledgeBase
     * @return the KnowledgeBase
     */
    KnowledgeBase getKnowledgeBase();

    /**
     * Get the current state of the working memory
     * @return the state
     * @see {@link org.ow2.jasmine.rules.api.DroolsWorkingMemoryState}
     */
    DroolsWorkingMemoryState getState();

    KnowledgeBuilder newKnowledgeBuilder();

}
