/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:RulesManagerModelLocator.as 2485 2008-09-30 14:14:35Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.manager.model.textual;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.ow2.jasmine.rules.manager.model.AbstractRulePackage;
import org.ow2.jasmine.rules.manager.model.IRulePackage;
import org.ow2.jasmine.rules.manager.service.IDroolsManager;


/**
 * A rule package object that can be edited by JASMINe's textual rule editor.
 *
 * Uses JAXB annotations for XML marshaling/unmarshaling
 * (see {@link IDroolsManager#exportRulePackageToXML(IRulePackage)}).
 *
 * @author Julien Nicoulaud
 */
@XmlRootElement(name = "textual-rule-package")
public class TextualRulePackage extends AbstractRulePackage
                                implements ITextualRulePackage {

    // =========================================================================
    // Properties
    // =========================================================================

    /**
     * Class version ID.
     */
    private static final long serialVersionUID = 1904696300611825640L;

    /**
     * The DRL source of the rule package.
     */
    @XmlTransient
    public String content;


    // =========================================================================
    // Getters & Setters
    // =========================================================================

    /**
     * Get the content.
     */
    @XmlElement(required = true)
    public String getContent() {
        return content;
    }

    /**
     * Set the content.
     */
    public void setContent(final String value) {
        content = value;
    }


    // =========================================================================
    // Utils
    // =========================================================================

    /**
     * Get the generated source in Drools syntax.
     */
    public String getDrlSource() {
        return content;
    }
}
