/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.manager.model.visual;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAnyElement;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlElementRefs;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.ow2.jasmine.rules.manager.model.AbstractRulePackageMember;

/**
 * A single rule object.
 *
 * @author Julien Nicoulaud
 * @see {@link IVisualRulePackage}
 */
@XmlRootElement(name = "rule")
public class Rule extends AbstractRulePackageMember
                  implements IRule {

    // =========================================================================
    // Properties
    // =========================================================================

    /**
     * Class version ID.
     */
    private static final long serialVersionUID = 1490850185807402023L;

    /**
     * The rule name.
     */
    @XmlTransient
    public String name = "";

    /**
     * The LHS (conditions).
     */
    @XmlTransient
    public List<ILHSElement> lhs = new ArrayList<ILHSElement>();

    /**
     * The RHS (actions).
     */
    @XmlTransient
    public List<IRHSElement> rhs = new ArrayList<IRHSElement>();

    /**
     * The variables defined in the rule.
     */
    @XmlTransient
    public List<String> variables = new ArrayList<String>();


    // =========================================================================
    // Getters and setters
    // =========================================================================

    /**
     * Set the rule name.
     */
    public void setName(final String value) {
        name = value;
    }

    /**
     * Get the rule name.
     */
    @XmlAttribute
    public String getName() {
        return name;
    }

    /**
     * Set the LHS.
     */
    public void setLHS(final List<ILHSElement> value) {
        lhs = value;
    }

    /**
     * Get the LHS.
     */
    @XmlElementWrapper(name = "lhs")
    @XmlAnyElement(lax=true)
    @XmlElementRefs({
        @XmlElementRef(name = "lhs-code-snippet", type = LHSCodeSnippet.class),
        @XmlElementRef(name = "logical-expression", type = LogicalExpression.class)
    })
    public List<ILHSElement> getLHS() {
        return lhs;
    }

    /**
     * Set the RHS.
     */
    public void setRHS(final List<IRHSElement> value) {
        rhs = value;
    }

    /**
     * Get the RHS.
     */
    @XmlElementWrapper(name = "rhs")
    @XmlAnyElement(lax=true)
    @XmlElementRefs({
        @XmlElementRef(name = "rhs-code-snippet", type = RHSCodeSnippet.class)
    })
    public List<IRHSElement> getRHS() {
        return rhs;
    }

    /**
     * Set the variables defined in the rule.
     */
    public void setVariables(List<String> value) {
        variables = value;
    }

    /**
     * Get the variables defined in the rule.
     */
    @XmlElementWrapper
    @XmlAnyElement
    public List<String> getVariables() {
        return variables;
    }


    // =========================================================================
    // Implementation of IRulePackageMember
    // =========================================================================

    /**
    * Get the generated source in Drools syntax.
    */
    public String getDrlSource() {
        String source = "";

        // Format the name
        source += "rule \"" + name + "\"";

        // Format conditions
        source += "\nwhen";
        for (ILHSElement lhsElement : lhs) {
            source += "\n    " + lhsElement.getDrlSource();
        }

        // Format actions
        source += "\nthen";
        for (IRHSElement rhsElement : rhs) {
            source += "\n    " + rhsElement.getDrlSource();
        }

        // Format the suffix
        source += "\nend";

        return source;
    }

}
