/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.manager.model.visual;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAnyElement;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlElementRefs;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.ow2.jasmine.rules.manager.model.AbstractRulePackage;
import org.ow2.jasmine.rules.manager.model.IRulePackage;
import org.ow2.jasmine.rules.manager.service.IDroolsManager;

/**
 * A rule package object that can be edited by JASMINe's visual rule editor.
 *
 * Uses JAXB annotations for XML marshaling/unmarshaling
 * (see {@link IDroolsManager#exportRulePackageToXML(IRulePackage)}).
 *
 * @author Julien Nicoulaud
 */
@XmlRootElement(name="visual-rule-package")
public class VisualRulePackage extends AbstractRulePackage
                               implements IVisualRulePackage {

    // =========================================================================
    // Variables
    // =========================================================================

    /**
     * Serial version ID.
     */
    private static final long serialVersionUID = 1L;

    /**
     * The rules list.
     */
    @XmlTransient
    public List<IRule> rules = new ArrayList<IRule>();

    /**
     * The imports list.
     */
    @XmlTransient
    public List<String> imports = new ArrayList<String>();


    // =========================================================================
    // Getters and setters
    // =========================================================================

    /**
     * Set the rules collection.
     */
    public void setRules(final List<IRule> value)
    {
        rules = value;
    }

    /**
     * Get the rules collection.
     */
    @XmlElementWrapper(name = "rules")
    @XmlAnyElement(lax = true)
    @XmlElementRefs({
        @XmlElementRef(name = "rule", type = Rule.class)
    })
    public List<IRule> getRules()
    {
        return rules;
    }

    /**
     * Set the imports.
     */
    public void setImports(List<String> value) {
        imports = value;
    }

    /**
     * Get the imports.
     */
    @XmlElementWrapper(name = "imports")
    @XmlAnyElement
    public List<String> getImports() {
        return imports;
    }


    // =========================================================================
    // Utils
    // =========================================================================

    /**
     * Get the generated source in Drools syntax.
     */
    public String getDrlSource() {
        String source = "";

        // Format the package statement
        source += "package " + getSystemName();

        // Format the imports section
        for (String importStatement : imports) {
            source += "import " + importStatement + ";\n";
        }

        // Format rules
        for (IRule rule : rules) {
            source += "\n\n" + rule.getDrlSource();
        }

        return source;
    }
}
